/***************************************************************************
 *   $Id: mscr.cpp,v 1.13 2017-12-13 11:16:52 giacomo Exp $
 *                                                                         *
 *   Copyright (C) 2009 by Fabio Asnicar  fabio.asnicar@elettra.trieste.it *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 *   In addition, as a special exception, the copyright holders give       *
 *   permission to link the code of this program with any edition of       *
 *   the Qt library by Trolltech AS, Norway (or with modified versions     *
 *   of Qt that use the same license as Qt), and distribute linked         *
 *   combinations including the two.  You must obey the GNU General        *
 *   Public License in all respects for all of the code used other than    *
 *   Qt.  If you modify this file, you may extend this exception to        *
 *   your version of the file, but you are not obligated to do so.  If     *
 *   you do not wish to do so, delete this exception statement from        *
 *   your version.                                                         *
 ***************************************************************************/

// cumbia
#include <cumbiapool.h>
#include <cuserviceprovider.h>
#include <cumacros.h>
#include <quapps.h>
#include <quwatcher.h>
#include <quwriter.h>
// cumbia

#include "mscr.h"
#include <QTimer>

mscr::mscr(CumbiaPool *cumbia_pool, QWidget *parent) : QWidget(parent) {
    // cumbia
    CuModuleLoader mloader(cumbia_pool, &m_ctrl_factory_pool, &m_log_impl);
    cu_pool = cumbia_pool;
    ui = new Ui::mscr;
    ui->setupUi(this, cu_pool, m_ctrl_factory_pool);
    foreach(QString s, QStringList() << "gbLightLevel2" << "attenuator2_groupBox"
            << "verSlit_groupBox" << "horSlit_groupBox")
        findChild<QGroupBox *>(s)->setVisible(false);

    // wait for mscr status and instrument list ready
    QTimer *tmr_checkConfigReady = new QTimer(this);
    tmr_checkConfigReady->setInterval(500);
    tmr_checkConfigReady->setObjectName("checkConfigTmr");
    connect(tmr_checkConfigReady, SIGNAL(timeout()), this, SLOT(checkConfigReady()));
    tmr_checkConfigReady->start();

    // mloader.modules() to get the list of loaded modules
    // cumbia

    if (QApplication::arguments().size() > 1) {
        QuWatcher *w = new QuWatcher(this, cumbia_pool, m_ctrl_factory_pool);
        w->attach(this, SLOT(onMscrStatus(const QVector<bool>&)));
        w->setSource("$1/MscrStatus");

        Qu1TWatcher *w1 = new Qu1TWatcher(this, cumbia_pool, m_ctrl_factory_pool);
        w1->attach(this, SLOT(onControllerType(const QString&)));
        w1->setSource("$1/ControllerType");

        Qu1TWatcher *w2 = new Qu1TWatcher(this, cumbia_pool, m_ctrl_factory_pool);
        w2->attach(&m_instruments);
        w2->setSource("$1->GetInstrumentList");

        Qu1TWatcher *llw = new Qu1TWatcher(this, cumbia_pool, m_ctrl_factory_pool);
        llw->attach(this, SLOT(onLightLevelReady(int)));
        llw->setSource("$1/LightLevel");

        // position
        QuWatcher *pos_w = new QuWatcher(this, cumbia_pool, m_ctrl_factory_pool);
        pos_w->attach(this, SLOT(onPositionReady(int)));
        // onPositionReady connects radio buttons
        pos_w->setSource("$1/Selector");

        ui->qio_polar1Attenuation->setSource("$1/Polar1Attenuation");

    } else {
        usage();
        exit(1);
    }
    resize(minimumSizeHint());
}

mscr::~mscr() {
}

void mscr::usage() {
    cout << endl;
    cout << "Usage:" << endl;
    cout << "  mscr device" << endl;
    cout << endl;
}

void mscr::initWidgets() {
    qDebug() << __PRETTY_FUNCTION__ << m_mscrStatus << m_instruments;
	
	// attenuator 1 && polarimeter
	ui->attenuator1_groupBox->setVisible(m_mscrStatus[1]);
	ui->qio_polar1Attenuation->setVisible(m_mscrStatus[1]);
	

    // light level (straight)
    ui->gbLightLevel2->setVisible(m_mscrStatus[3]);
    ui->attenuator2_groupBox->setVisible(m_mscrStatus[3]);//2

    if(ui->gbLightLevel2->isVisible()) {
        ui->qulLightLevel2->setSource("$1/LightLevel2");
        ui->qio_polar2Attenuation->setSource("$1/Polar2Attenuation");

        QuWatcher *ll2w = new QuWatcher(this, cu_pool, m_ctrl_factory_pool);
        ll2w->attach(this, SLOT(onLightLevel2Ready(int)));
        ll2w->setSource("$1/LightLevel2");
    }


    ui->verSlit_groupBox->setEnabled(m_mscrStatus[6]);//5
    ui->verSlit_groupBox->setVisible(m_mscrStatus[5]);//4
    if(ui->verSlit_groupBox->isVisible())
        ui->qioVSlit->setSource("$1/VerSlitPosition");

    ui->stepperHorStatus_groupBox->setVisible(m_mscrStatus[5]);//4
    ui->horSlit_groupBox->setEnabled(m_mscrStatus[7]);//4
    ui->horSlit_groupBox->setVisible(m_mscrStatus[5]);//4
    if(ui->horSlit_groupBox->isVisible())
        ui->qioHSlit->setSource("$1/HorSlitPosition");

    // Status tab
	
	ui->polar1Status_groupBox->setVisible(m_mscrStatus[1]);
	
    ui->polar2Status_groupBox->setVisible(m_mscrStatus[3]);//2
    if(ui->polar2Status_groupBox->isVisible())
        ui->qutable_polar2Status->setSource("$1/Polar2Status");

    ui->stepperVerStatus_groupBox->setVisible(m_mscrStatus[5]);//4
    if(ui->stepperVerStatus_groupBox->isVisible())
        ui->qutable_stepperVerStatus->setSource("$1/StepperVerStatus");

    ui->stepperHorStatus_groupBox->setVisible(m_mscrStatus[5]);//4
    if(ui->stepperHorStatus_groupBox->isVisible())
        ui->qutable_stepperHorStatus->setSource("$1/StepperHorStatus");


    ui->rbp0->setText(QString(m_instruments[0]));
    ui->rbp1->setText(QString(m_instruments[1]));
    if (m_instruments.size() > 2) {
        ui->rbp2->setText(QString(m_instruments[2]));
    } else {
        ui->rbp2->setHidden(true);
    }
    if (m_instruments.size() > 3) {
        ui->rbp3->setText(QString(m_instruments[3]));
    } else {
        ui->rbp3->setHidden(true);
    }
    if (m_instruments.size() > 4) {
        ui->rbp4->setText(QString(m_instruments[4]));
    } else {
        ui->rbp4->setHidden(true);
    }
}

void mscr::onControllerType(const QString &contr_type) {
    if (!contr_type.startsWith("IFC-B")){
        ui->tabWidget->removeTab(3);
    }
}

void mscr::checkConfigReady() {
    printf("mscr::checkConfigReady() enter\n");
    // init widgets when both instrument list and mscr status are ready
    if(m_instruments.size() > 0 && this->m_mscrStatus.size() > 0) {
        delete findChild<QTimer* >("checkConfigTmr");
        initWidgets();
    }
}

void mscr::onLightLevelReady(int ll) {
    ui->lightLevel_comboBox->setCurrentIndex(ll);
    connect(ui->lightLevel_comboBox,   SIGNAL(currentIndexChanged(int)), this, SLOT(setLightLevel(int)));
}

void mscr::onLightLevel2Ready(int ll) {
    ui->lightLevel_comboBox_2->setCurrentIndex(ll);
    connect(ui->lightLevel_comboBox_2,   SIGNAL(currentIndexChanged(int)), this, SLOT(setLightLevel2(int)));
}

void mscr::onPositionReady(int position) {
    QFont fontNorma, fontBold;
    fontNorma.setBold(false);
    fontBold.setBold(true);
    for(int i = 0; i < 5; i++) {
        QRadioButton *rb = ui->selector_groupBox->findChild<QRadioButton *>(QString("rbp%1").arg(i));
        rb->disconnect();
        position == i ? rb->setFont(fontBold) : rb->setFont(fontNorma);
        rb->setChecked(position == i);
        connect(rb, SIGNAL(toggled(bool)), this, SLOT(onPositionRbChecked(bool)));
    }
}

void mscr::onMscrStatus(const QVector<bool>& s) {
    m_mscrStatus = std::move(s);
    ui->verSlit_groupBox->setEnabled(m_mscrStatus[5]);
    ui->horSlit_groupBox->setEnabled(m_mscrStatus[6]);
}

void mscr::onPositionRbChecked(bool c) {
    if(c) {
        printf("mscr::onPositionRbChecked %s checked %d\n", qstoc(sender()->objectName()), c);
        int p = sender()->objectName().remove("rbp").toInt();
        QuWriter *w = new QuWriter(this, cu_pool, m_ctrl_factory_pool);
        w->setAutoDestroy(true);
        w->setTarget("$1/Selector");
        w->execute(p);
    }
}

void mscr::setLightLevel(int newLevel) {
    QuWriter *w = new QuWriter(this, cu_pool, m_ctrl_factory_pool);
    w->setAutoDestroy(true);
    w->setTarget("$1/LightLevel");
    w->execute(newLevel);
}

void mscr::setLightLevel2(int newLevel) {
    QuWriter *w = new QuWriter(this, cu_pool, m_ctrl_factory_pool);
    w->setAutoDestroy(true);
    w->setTarget("$1/LightLevel2");
    w->execute(newLevel);
}


